<?php

namespace Drupal\psa_oecd_publishing\Form;

use Drupal\Core\Entity\EntityFieldManager;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\psa_oecd_publishing\Controller\OecdMapperController;
use Drupal\psa_oecd_publishing\Controller\OecdPublishingController;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a list of recalls to be published to the OECD GlobalRecalls API.
 */
class OecdPublishingRecallsList extends FormBase {

  /**
   * Creates a OecdPublishingSettingsForm instance.
   *
   * @param \Drupal\Core\Entity\EntityFieldManager $entityFieldManager
   *   The entity_field.manager service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity_type.manager service.
   * @param \Drupal\psa_oecd_publishing\Controller\OecdPublishingController $oecdPublisher
   *   The psa_oecd_publishing.controller service.
   */
  public function __construct(
    protected EntityFieldManager $entityFieldManager,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected OecdPublishingController $oecdPublisher,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): object {
    return new static(
      $container->get('entity_field.manager'),
      $container->get('entity_type.manager'),
      $container->get('psa_oecd_publishing.controller'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'psa_oecd_publishing_publish_needed_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('psa_oecd_publishing.settings');
    $jurisdiction = $config->get('recall_jurisdiction');

    if (!$jurisdiction) {
      $this->messenger()->addWarning($this->t('OECD GlobalRecalls API module is not configured.'));
      return $form;
    }

    // Display a table of nodes that need to be published.
    $nodes = $this->oecdPublisher->getNodes();
    $rows = [];
    foreach ($nodes as $node) {
      foreach ($this->oecdPublisher->getRecallLanguages($node) as $langcode) {
        $translated_node = $node->getTranslation($langcode);

        $recall = $this->oecdPublisher->getRecall($translated_node);
        $operation = $this->oecdPublisher->getRecallOperation($recall, psa_oecd_publishing_node_is_published($translated_node));

        // Skip nodes that do not need any action.
        if (!$operation) {
          $this->logger('psa_oecd_publishing')->notice('Node @nodeId needs no operation; skipping.', ['@nodeId' => $translated_node->id()]);
          $this->oecdPublisher->updateNodeStatus($translated_node, TRUE);
          continue 2;
        }

        $row = [
          $translated_node->toLink(),
          $langcode,
          $this->oecdPublisher->getRecallId($translated_node),
          OecdMapperController::getOperationName($operation),
        ];
        $rows[] = $row;
      }
    }

    $form['recalls_table'] = [
      '#type' => 'table',
      '#empty' => $this->t('No recalls to publish.'),
      '#header' => [
        $this->t('Recall'),
        $this->t('Language'),
        $this->t('Recall ID'),
        $this->t('API operation'),
      ],
      '#rows' => $rows,
    ];

    $form['nids'] = [
      '#type' => 'value',
      '#value' => array_keys($nodes),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Publish these to OECD GlobalRecalls portal'),
      // Disable when there are no nodes to publish.
      '#disabled' => !$rows,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    // Load the nodes that were displayed on the form screen.
    $nids = $form_state->getValue('nids');
    $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($nids);
    // Publish the nodes (insert, update, delete).
    [$isSuccess, $status] = $this->oecdPublisher->publish(FALSE, $nodes);
    // Output status.
    if ($isSuccess) {
      $this->messenger()->addStatus($this->t(
        'Publishing to OECD completed. Nodes processed: @total. Succeeded: @passed. Failed: @failed. Check logs for details.',
        [
          '@total' => $status['total'],
          '@passed' => $status['passed'],
          '@failed' => $status['failed'],
        ]
      ));
    }
    else {
      $this->messenger()->addError('There was an error processing. Check logs for details.');
    }
  }

}
